package com.ejie.ab61.control.solicitudes;

import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.model.solicitudes.Configuracion;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.solicitudes.ConfiguracionService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ConfiguracionController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/configuracion")

public class ConfiguracionController {

	private static final Logger logger = LoggerFactory.getLogger(ConfiguracionController.class);

	@Autowired
	private ConfiguracionService configuracionService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idConfiguracion
	 *            Integer
	 * @return Configuracion Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idConfiguracion}", method = RequestMethod.GET)
	public @ResponseBody Configuracion get(@PathVariable Integer idConfiguracion) {
		Configuracion configuracion = new Configuracion();
		configuracion.setIdConfiguracion(idConfiguracion);
		configuracion = this.configuracionService.find(configuracion);
		ConfiguracionController.logger.info("[GET - findBy_PK] : Obtener Configuracion por PK");
		return configuracion;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterConfiguracion
	 *            Configuracion Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Configuracion> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Configuracion> getAll(@ModelAttribute Configuracion filterConfiguracion) {
		ConfiguracionController.logger.info("[GET - find_ALL] : Obtener Configuracion por filtro");
		return this.configuracionService.findAll(filterConfiguracion, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param configuracion
	 *            Configuracion Bean que contiene la informacion a modificar.
	 * @return Configuracion Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Configuracion edit(@RequestBody Configuracion configuracion, Usuario usuario) {

		Date fechaHoy = new Date();
		configuracion.setFechaUltmodificacion(fechaHoy);
		configuracion.setUsuarioUltmodificacion(usuario.getNombreUsuario());

		Configuracion configuracionAux = this.configuracionService.update(configuracion);
		ConfiguracionController.logger.info("[PUT] : Configuracion actualizado correctamente");
		return configuracionAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param configuracion
	 *            Configuracion Bean que contiene la informacion con la que se
	 *            va a crear el nuevo registro.
	 * @return Configuracion Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Configuracion add(@RequestBody Configuracion configuracion, Usuario usuario) {

		Date fechaHoy = new Date();
		configuracion.setFechaRegistro(fechaHoy);
		configuracion.setUsuarioRegistro(usuario.getNombreUsuario());

		Configuracion configuracionAux = this.configuracionService.add(configuracion);
		ConfiguracionController.logger.info("[POST] : Configuracion insertado correctamente");
		return configuracionAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idConfiguracion
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Configuracion Bean eliminado.
	 */
	@RequestMapping(value = "/{idConfiguracion}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Configuracion remove(@PathVariable Integer idConfiguracion) {
		Configuracion configuracion = new Configuracion();
		configuracion.setIdConfiguracion(idConfiguracion);
		this.configuracionService.remove(configuracion);
		ConfiguracionController.logger.info("[DELETE] : Configuracion borrado correctamente");
		return configuracion;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		ConfiguracionController.logger.info("[GET - View] : configuracion");
		return "configuracion";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterConfiguracion
	 *            Configuracion Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Configuracion> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Configuracion> filter(
			@RequestJsonBody(param = "filter") Configuracion filterConfiguracion,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ConfiguracionController.logger.info("[POST - filter] : Obtener Configuracions");
		return this.configuracionService.filter(filterConfiguracion, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterConfiguracion
	 *            Configuracion Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchConfiguracion
	 *            Configuracion Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Configuracion> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Configuracion>> search(
			@RequestJsonBody(param = "filter") Configuracion filterConfiguracion,
			@RequestJsonBody(param = "search") Configuracion searchConfiguracion,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ConfiguracionController.logger.info("[POST - search] : Buscar Configuracions");
		return this.configuracionService.search(filterConfiguracion, searchConfiguracion, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterConfiguracion
	 *            Configuracion Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param = "filter") Configuracion filterConfiguracion,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ConfiguracionController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Configuracions");
		this.configuracionService.removeMultiple(filterConfiguracion, jqGridRequestDto, false);
		ConfiguracionController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterConfiguracion
	 *            Configuracion Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Configuracion>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Configuracion>> jerarquia(
			@RequestJsonBody(param = "filter") Configuracion filterConfiguracion,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ConfiguracionController.logger.info("[POST - jerarquia] : Obtener Configuracions jerarquia");
		return this.configuracionService.jerarquia(filterConfiguracion, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterConfiguracion
	 *            Configuracion Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Configuracion>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Configuracion>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Configuracion filterConfiguracion,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		ConfiguracionController.logger.info("[POST - jerarquia] : Obtener Configuracions jerarquia - Hijos");
		return this.configuracionService.jerarquiaChildren(filterConfiguracion, jqGridRequestDto);
	}

	/**
	 * Método que comprueba si existe más de un registro con el mismo
	 * idConfigEventos y codProcedureId en la tabla de configuración
	 * 
	 * @param codProcedureId
	 *            Integer
	 * @param idConfigEventos
	 *            Integer
	 * @return Integer Número de registros encontrados.
	 */
	@RequestMapping(value = "comprobarConfigEventosRepetido/{codProcedureId}/{idConfigEventos}", method = RequestMethod.GET)
	public @ResponseBody Integer comprobarConfigEventosRepetido(@PathVariable String codProcedureId,
			@PathVariable Integer idConfigEventos) {

		Integer numeroRegistros = this.configuracionService.comprobarConfigEventosRepetido(codProcedureId,
				idConfigEventos);

		ConfiguracionController.logger
				.info("[GET - comprobarConfigEventosRepetido] : Número de registros - " + numeroRegistros);
		return numeroRegistros;
	}

}
